module.exports = function extendDiscordGuild(Guild) {
    class DiscordUtilsGuild extends Guild {
        constructor(client, data) {
            super(client, data);

            /**
             * Find one channel by name or id
             * @param {String} name - Channel name
             * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
             * @param {Object[]} settings - Options for this function
             * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
             * @return {Object} channel
             */
            this.channels.findOne = (name, options = {}, settings = {includes: false}) => {
                return new Promise(async (resolve, reject) => {
                    //validate input
                    if (typeof name !== "string" || !name)
                        return reject(new Error(`name cannot be null or typeof ${typeof name}, String expected!`))

                    if (typeof options !== "object" && typeof options !== "undefined")
                        return reject(new Error(`options cannot be typeof ${typeof options}, Object expected!`))

                    //get channel by id
                    if (!name.match(new RegExp('^[a-zA-Z]+$'))) return resolve(await this.channels.cache.get(name))

                    //find channel by name
                    return resolve(this.channels.cache.find(c => {
                        let channelName = c.name
                        if (!settings.caseSensitive) channelName = channelName.toLowerCase()

                        if (options.type && settings.includes) return c.type === options.type && channelName.includes(name.toLowerCase())
                        if (!options.type && settings.includes) return channelName.includes(name.toLowerCase())
                        if (options.type && !settings.includes) return c.type === options.type && channelName === name.toLowerCase()

                        return channelName === name.toLowerCase()
                    }))
                })
            },

                /**
                 * Find all channel by name or id
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 * @return {Collection} channels
                 */
                this.channels.findAll = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        //validate input
                        if (typeof name !== "string" || !name)
                            return reject(new Error(`name cannot be null or typeof ${typeof name}, String expected!`))

                        if (typeof options !== "object" && typeof options !== "undefined")
                            return reject(new Error(`options cannot be typeof ${typeof options}, Object expected!`))

                        //get channel by id
                        if (!name.match(new RegExp('^[a-zA-Z]+$'))) return resolve(await this.channels.cache.get(name))

                        let channels = this.channels.cache.filter(c => {
                            let channelName = c.name
                            if (!settings.caseSensitive) channelName = channelName.toLowerCase()

                            if (options.type && settings.includes) return c.type === options.type && channelName.includes(name.toLowerCase())
                            if (!options.type && settings.includes) return channelName.includes(name.toLowerCase())
                            if (options.type && !settings.includes) return c.type === options.type && channelName === name.toLowerCase()

                            return channelName === name.toLowerCase()
                        })

                        //find channel by name
                        return resolve(channels)
                    })
                },

                /**
                 * Find or Create for discord channels
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 * @return {Object} channel
                 */
                this.channels.findOrCreate = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        this.channels.findOne(name, options, settings).then(async channel => {
                            if (!channel) return resolve(await this.channels.create(name, options));
                            return resolve(channel)
                        }).catch(reject)
                    })
                },

                /**
                 * Find and Delete for discord channels
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 * @return {Boolean}
                 */
                this.channels.findAndDelete = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        this.channels.findOne(name, options, settings).then(async channel => {
                            if (!channel) return resolve(false);
                            await channel.delete().catch(reject)
                            return resolve(true)
                        }).catch(reject)
                    })
                },


                /**
                 * Find one role by name or id
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 * @return {Object} role
                 */
                this.roles.findOne = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        //validate input
                        if (typeof name !== "string" || !name)
                            return reject(new Error(`name cannot be null or typeof ${typeof name}, String expected!`))

                        if (typeof options !== "object" && typeof options !== "undefined")
                            return reject(new Error(`options cannot be typeof ${typeof options}, Object expected!`))

                        //get role by id
                        if (!name.match(new RegExp('^[a-zA-Z]+$'))) return resolve(await this.roles.cache.get(name))

                        //find role by name
                        return resolve(this.roles.cache.find(r => {
                            let roleName = r.name

                            if (settings.includes) return roleName.includes(name.toLowerCase())
                            return roleName === name.toLowerCase()
                        }))
                    })
                },

                /**
                 * Find one role by name or id
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 * @return {Collection} roles
                 */
                this.roles.findAll = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        //validate input
                        if (typeof name !== "string" || !name)
                            return reject(new Error(`name cannot be null or typeof ${typeof name}, String expected!`))

                        if (typeof options !== "object" && typeof options !== "undefined")
                            return reject(new Error(`options cannot be typeof ${typeof options}, Object expected!`))

                        //get role by id
                        if (!name.match(new RegExp('^[a-zA-Z]+$'))) return resolve(await this.roles.cache.get(name))

                        //find role by name
                        return resolve(this.roles.cache.filter(r => {
                            let roleName = r.name

                            if (settings.includes) return roleName.includes(name.toLowerCase())
                            return roleName === name.toLowerCase()
                        }))
                    })
                },

                /**
                 * Find or Create for discord roles
                 * @param {String} name - Channel name
                 * @param {Object} options - Role options from discord.js https://discord.js.org/#/docs/main/stable/class/RoleManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].nameIncludes - Use the string function .includes instead of comparing the names directly
                 */
                this.roles.findOrCreate = (name, options = {}, settings = {nameIncludes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        this.roles.findOne(name, options, settings).then(async role => {
                            //return role
                            if (role) return resolve(role)

                            //set default options
                            if (!options.data) options.data = {}
                            if (!options.data.name) options.data.name = name
                            if (!options.data.color) options.data.color = Math.floor(Math.random() * 16777215).toString(16)

                            //return new created role
                            return resolve(await this.roles.create(options));
                        }).catch(reject)
                    })
                },

                /**
                 * Find and Delete for discord roles
                 * @param {String} name - Channel name
                 * @param {Object} options - Channel options from discord.js https://discord.js.org/#/docs/main/stable/class/GuildChannelManager?scrollTo=create
                 * @param {Object[]} settings - Options for this function
                 * @param {Object[]} settings[].includes - Use the string function .includes instead of comparing the names directly
                 */
                this.roles.findAndDelete = (name, options = {}, settings = {includes: false}) => {
                    return new Promise(async (resolve, reject) => {
                        this.roles.findOne(name, options, settings).then(async role => {
                            if (!role) return resolve(false);
                            await role.delete().catch(reject)
                            return resolve(true)
                        }).catch(reject)
                    })
                }
        }
    }

    console.log(DiscordUtilsGuild)
    return DiscordUtilsGuild;
}